/*****************            jack .c     *********************************
 Simple jackknife program for computation of jackknife errors of data.
 The jackknife function can be incerted to any program.
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <libgen.h>

#define  MAXDAT 1000000
#define  STRLEN 200
char prog[STRLEN];
int JACK,maxdat;

void jackknife(int,int,double *,double *,double *,double *,double *);
void get_the_options(int ,char **),usage(char **),locerr(char *);
int main(int argc,char **argv){
  int    ndat=0;
  double O,dO,chi,dchi;
  double *x;
  strcpy(prog,(char *)basename(argv[0]));
  maxdat=-1;JACK=10;
  get_the_options(argc,argv);
  if( maxdat <= 0 ) maxdat = MAXDAT;
  x   = (double *) calloc((size_t) maxdat,(size_t) sizeof(double));
  /*Read data from stdin as long as it fits the array x[].
    ndat counts the number of data.*/
  while( scanf("%lg",&x[ndat])==1 ){
    ndat++;
    if(ndat == maxdat){//we give user a warning if we ignore some data
      fprintf(stderr,"# %s: Warning: Read ndat= %d. Reached the limit %d\n",
	      prog,ndat,maxdat);
      break;           //break the while loop
    }
  }
  jackknife(ndat,JACK,x,&O,&dO,&chi,&dchi);//calculate....
  printf("# NDAT = %d data. JACK = %d groups\n",ndat,JACK);//...and print
  printf("# <o>, chi= (<o^2>-<o>^2)\n");
  printf("# <o> +/- err                             chi +/- err\n");
  printf("%.15lg %.15lg %.15lg %.15lg\n",O,dO,chi,dchi);
}
/****************************************************************/
/* jackknife routine: you can use it in any of your programs....*/
/****************************************************************/
void jackknife(int ndat, int jack, double *x, 
	  double *avO, double *erO, double *avchi, double *erchi){

  int i,j,binw,bin;
  double *O,*chi;
  O    = (double *) calloc((size_t) jack  ,(size_t) sizeof(double));
  chi  = (double *) calloc((size_t) jack  ,(size_t) sizeof(double));
  binw = ndat/jack;
  if(binw<1)locerr("jackknife: binw < 1");
  ndat = binw * jack; //only full bins considered
  /* average value   */
  for(i=0;i<ndat;i++)
    for(j=0;j<jack;j++){
      if( (i/binw) != j ){ //then we add this data point to the bin
	O [j] += x[i];
      }
    }
  for(j=0;j<jack;j++) O  [j] /= (ndat-binw); //average in each bin
  /* susceptibility: */
  for(i=0;i<ndat;i++)
    for(j=0;j<jack;j++){
      if( (i/binw) != j ){ //then we add this data point to the bin
	chi[j] += (x[i]-O[j])*(x[i]-O[j]);
      }
    }
  for(j=0;j<jack;j++) chi[j] /= (ndat-binw); //average in each bin
  /* Compute averages: */
  *avO = 0.0; *avchi = 0.0;
  for(j=0;j<jack;j++){
    *avO += O[j]; *avchi += chi[j];
  }
  *avO /= jack; *avchi /= jack;
  /* Compute errors:   */
  *erO = 0.0; *erchi = 0.0;
  for(j=0;j<jack;j++){
    *erO   += (O  [j]-*avO  )*(O  [j]-*avO  );
    *erchi += (chi[j]-*avchi)*(chi[j]-*avchi);
  }
  *erO = sqrt(*erO); *erchi = sqrt(*erchi);
  free(O);free(chi);//release memory back to the system
}
/****************************************************************/
void locerr( char *errmes ){
  fprintf(stderr,"%s: %s Exiting....\n",prog,errmes);
  exit(1);
}

#define OPTARGS  "?hj:d:"
void get_the_options(int argc,char **argv){

  int c,errflg = 0;
  while (!errflg && (c = getopt(argc, argv, OPTARGS)) != -1){
    switch(c){
    case 'j':
      JACK = atoi(optarg);
      break;
    case 'd':
      maxdat = atoi(optarg);
      break;
    case 'h':
      errflg++;/*call usage*/
      break;
    default:
      errflg++;
    }/*switch*/
    if(errflg) usage(argv);
  }/*while...*/
}/*get_the_options()*/

void usage(char **argv){
  fprintf(stderr,"\
Usage: %s  [options] <file>\n\
       -j  : No. jack groups Def. 10\n\
       -d  : Give the maximum number of data points read.Def. %d\n\
Computes <o>, chi= (<o^2>-<o>^2)\n\
Data is in one column from stdin.\n",prog,MAXDAT);
  exit(1);
}/*usage()*/
