/*********************** gausran.c *****************************/
// Program to produce gaussian distribution of pseudo-random numbers.
// Compile as:
// % cc gausran.c -o gausran -lm
// Output:
// u_i  g_i   c_i
// u_i:uniform distribution g_i:gaussian distribution c_i:Cauchy distribution
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#define PI  3.1415926535897932385
#define PI2 6.2831853071795864769

double drandom(), gausran(), cauchyran();
int main(int argc, char **argv){
  int i,N;
  if(argc == 1) N=1000;
  else          N=atoi(argv[1]);

  for(i=1;i<=N;i++)
    printf("%f %f %f\n",drandom(),gausran(),cauchyran());
}

/************************  gausran() *****************************/
// Returns a gausian distributed random number with sigma = 1
// Each calculation produces 2 random numbers. One is saved for the
// next call.
double gausran(){
  double r,phi;
  static int    new = 1;
  static double x; // saved random number
  if(new){
    new =  0;
    phi =  PI2*drandom();
    if( (r=drandom()) == 0.0) r=drandom();//Check for 0. Hope not two in a row
    r   =  sqrt(-2.0 * log(r));
    x   =  r * cos(phi);
    return r * sin(phi);
  }
  else{
    new = 1;
    return x;
  }
}
/************************  cauchyran() ***************************/
double cauchyran(){
  return tan(PI2 * drandom()); 
}


/************************  drandom() *****************************/
#define a 16807
#define m 2147483647
#define q 127773
#define r 2836
#define conv (1.0/(m-1))

static int seed = 323412; 
double drandom(){
  long l;

  l    = seed/q;
  seed = a*(seed-q*l) - r*l;
  if(seed < 0) seed +=m;
  return conv*(seed-1);
}

