#!/bin/tcsh -f 

# Simple shell script to analyze data from the program nrrw2.c
# To see data run as:
# % cc -O2 nrrw2.c -o nrrw2
# % nrrw2-anal.csh  <measure> > nrrw2-anal.out
# % grep ^R2 nrrw2-anal.out
# % grep ^Al nrrw2-anal.out
# etc. etc.
# To plot results see gnuplot script nrrw2-anal-plot and use as:
# % gnuplot
# gnuplot> load "nrrw2-anal-plot"
set Nwalk = 1000
# L = 6*sqrt(N)
# set Ns    = (32 64 128 256 512 1024 2048 4096 8192 16384 32768 65536)
# set Ls    = (34 49  68  97 136  193  272  385  544   769  1087  1537)
set Ns    = (131072 262144 524288 1048576 2097152 4194304 8388608)
set Ls    = (  2173   3073   4345    6145    8689   12289   17378)
# set Ns  = ($Ns 131072 262144 524288 1048576 2097152 4194304 8388608 16777216)
# set Ls  = ($Ls   2173   3073   4345    6145    8689   12289   17378    24577)
# set Ns    = (32 64 128 256 512 1024 2048 4096)
# set Ls    = (34 49  68  97 136  193  272  385)
# set Ns    = (32 64 128 256 512)
# set Ls    = (34 49  68  97 136)

set prog = `basename $0`
if( $#argv != 1 )then
 echo "Usage: $prog <measure>"   > /dev/stderr
 echo "       <measure>=0 (no geom measurements), 1 (all measurements)" > /dev/stderr
 exit(1)
endif

set measure = $argv[1] # 0 no geometry measurements or 1 for all measurements
foreach N ($Ns)
 set L = $Ls[1]
 shift Ls
 set dat = nrrw2.$N.dat
 (/usr/bin/time ./nrrw2 $Nwalk $N $L $measure) >>& $dat
 gzip -f $dat
 set dat = $dat.gz
 echo -n "R2  $N  " # Compute 
 zgrep ^R    $dat | awk '{print $3}' | ./average
 echo -n "x   $N  "
 zgrep ^R    $dat | awk '{print $4}' | ./average
 echo -n "y   $N  "
 zgrep ^R    $dat | awk '{print $5}' | ./average
 echo -n "n   $N  "
 zgrep ^nret $dat | awk '{print $2}' | ./average
 zgrep ^Al $dat >& /dev/null
 if($status) goto skip_N
 echo -n "Al  $N  "
 set A1 = (`zgrep ^Al $dat | awk '{print $4}' | ./average`)
 set A2 = (`zgrep ^Al $dat | awk '{print $5}' | ./average`)
 set l0 = (`zgrep ^Al $dat | awk '{print $6}' | ./average`)
 set l1 = (`zgrep ^Al $dat | awk '{print $7}' | ./average`)
 set l2 = (`zgrep ^Al $dat | awk '{print $8}' | ./average`)
 set l3 = (`zgrep ^Al $dat | awk '{print $9}' | ./average`)
 echo "$A1   $A2   $l0   $l1   $l2   $l3"
skip_N:
end
