/************************  nrrw2.c *********************************/
/*
 Compile with:
 gcc -O2     nrrw2.c -o nrrw2
 icc -O3 -xK nrrw2.c -o nrrw2 (PIII - gcc is faster)
 icc -O3 -xW nrrw2.c -o nrrw2 (PIV)
 */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

/* 
   Nwalk: Number of random walks generated.
   N    : Length of random walks
   L    : Linear size of lattice and L2 = L*L  
 */
int    Nwalk, N, L, L2;
int    *h; /* The frequency function array h[L2]: How many times each 
              point is visited by the random walker */
int    R;     /*Final position of walker            */
int    iwalk; /*Random walk counter                 */
int    *hav;  /*Average h[r]                        */
int    measure_all;
void   init(int, char**),end();
void   generate_a_walk(),measure();
double measA1(); /*Area and length functions        */
void   measA2l1(double *,double *),measl2l3(double *,double *);
double drandom();
void   init_random();


/************************  main   () *****************************/
int main(int argc, char **argv){
  init(argc,argv); /*initialize program*/
  for(iwalk=1;iwalk<=Nwalk;iwalk++){
    generate_a_walk();
    measure();
  }
  end();
} //main()


/************************  init   () *****************************/
void init(int argc, char **argv){
  int i;
  if( argc != 5 ){
    fprintf(stderr,"Usage: %s <Nwalk> <N> <L> <measure=0,1>\n",basename(argv[0]));
    exit(1);
  }
  Nwalk       = atoi(argv[1]);
  N           = atoi(argv[2]);
  L           = atoi(argv[3]);
  measure_all = atoi(argv[4]);
  if( L%2 == 0 ) L++;  //we accept odd L only
  L2    = L*L;
  printf("# Size of lattice: L= %d  L2= %d\n",L,L2);
  printf("# Nwalk= %d NRRW random walks with Nstep= %d steps\n",Nwalk,N);
  /*Allocate memory for the h[L2] array */
  h   = (int *)malloc((size_t) (L2*sizeof(int)));
  if(h  ==NULL){fprintf(stderr,"nrrw2: Not enough memory!\n");exit(1);}
  hav = (int *)malloc((size_t) (L2*sizeof(int)));
  if(hav==NULL){fprintf(stderr,"nrrw2: Not enough memory!\n");exit(1);}
  for(i=0;i<L2;i++) {h[i] = 0;hav[i] = 0;} /*Init to zero*/
  init_random();/*Initialize random number generator*/
}

/************************  generate_a_walk() *********************/
void generate_a_walk(){
  int i,ir,r,r0;
  int dir;
  int boing; //count hits at the edge of lattice
  int nret ; //count no. of comeback times
  boing=0; nret=0;
  for(i=0;i<L2;i++) h[i] = 0;
  /*r is the position of the walker on the lattice*/
  r = L2/2; /*We place walker at the center of lattice (L2 is odd!)*/
  r0= r;
  dir=(int)(drandom()*4);/*dir is the stored direction of the previous step*/
  for(i=0;i<N;i++){ /*N steps foreach walk*/
    /*3 possible choices: (dir-1)%4,(dir)%4,(dir+1)%4*/
    ir=dir+3+(int)(drandom()*3);//ir always >0 for % operation, we add 4
    ir %= 4;
    dir = ir;//Stored for next move
    //The four directions are now stored counterclockwise:
    switch(ir){
    case 0: // Move in +X direction
      if((++r )>=L2){r -= L2;boing++;}//carefull ++r not r++!
      h[r]++;
      break;
    case 2: // Move in -X direction
      if((--r )< 0 ){r += L2;boing++;}//carefull --r not r--!
      h[r]++;
      break;
    case 1: // Move in +Y direction
      if((r+=L)>=L2){r -= L2;boing++;} 
      h[r]++;
      break;
    case 3: // Move in -Y direction
      if((r-=L)< 0 ){r += L2;boing++;} 
      h[r]++;
      break;
    }//switch(ir)
    //printf("w %d %d %d\n",iwalk,i,r);//if you wish to print the path
    if(r == r0) nret++;
  }//for(i=0;i<N;i++)
  R = r;
  printf("boing= %d  iwalk= %d\n",boing,iwalk);
  printf("nret= %d\n",nret);
}//generate_a_walk
/************************  measure() *****************************/
void measure(){
  int x,y; //translate r-position to x,y-position on the lattice
  int i,Lh;
  double xr,yr;
  double l0,l1,l2,l3,A0,A1,A2;
  double R2;

  Lh = L/2;
  /* Final position x,y,R2:        */
  y  = R/L  ; x  = R - y*L;
  y -= Lh   ; x -= Lh;//origin at the center of lattice
  yr = (double)y; xr = (double)x;
  R2 = xr*xr+yr*yr;
  printf  ("R %d %f %d %d %d\n",iwalk,R2,x,y,R);

  /* Area and length measurements: */
  if(measure_all){
    A0= (double)N; l0 = R2;
    A1 = measA1(); measA2l1(&A2,&l1); measl2l3(&l2,&l3);
    printf("Al %d %f %f %f %f %f %f %f\n",iwalk,A0,A1,A2,l0,l1,l2,l3);
  }//if(measure_all)
  /* Average histogram             */
  //  if(measure_all){
  //  for(i=0;i<L2;i++){
  //    hav[i] += h[i];
  //    if(iwalk <= 10 ){
  //	y=i/L;x=i-y*L;x-=Lh;y-=Lh;//if you wish to print all hs'
  //	printf("hst %d %d %d %d\n",iwalk,x,y,h[i]);
  //    }//if(iwalk <= 10 )
  //  }//for(i=0;i<L2;i++)
  // }//if(measure_all)
  fflush(stdout);
}//measure

/**** Area and length computation functions  *********************/
/**** A1 = # of sites that rw has visited    ****/
double measA1(){
  int icount,i;
  icount=0;
  for(i=0;i<L2;i++)
    if(h[i] > 0 ) icount++;
  return (double)icount;
}//A1()

/* A2 = # of sites that are internal to region rw has visited    */
/* l1 = # of boundary sites, i.e. at least one nn not visited    */
void measA2l1(double *A2, double *l1){
  int icountA,icountl,i,nn,sum;
  icountA = 0; icountl = 0;
  for(i=0;i<L2;i++){
    sum = 0;
    if(h[i]>0){
      /*Visit 4 neighbors. If all visited then site is internal*/
      if((nn=i+1)>= L2) nn-=L2; if( h[nn] > 0 ) sum++ ;
      if((nn=i-1)<  0 ) nn+=L2; if( h[nn] > 0 ) sum++ ;
      if((nn=i+L)>= L2) nn-=L2; if( h[nn] > 0 ) sum++ ;
      if((nn=i-L)<  0 ) nn+=L2; if( h[nn] > 0 ) sum++ ;
      if( sum == 4    ) icountA++;
      else              icountl++;
    }//if(h[i]>0)
  }//for(i=0;i<L2;i++)
  *A2 = (double)icountA; *l1 = (double)icountl;
}//A2l1()

/*l2 is the average of the distances squared                 */
/*l3 is the max distance squared between points              */
/*TODO: l4: as l2 but weigh each point by multiplicity h[i]  */
void measl2l3(double *l2, double *l3){
  int i, j;
  int xi, yi, xj, yj;
  double norm2;
  double xij, yij, rij;
  double len2, len3;

  len2=0.0;len3=0.0;norm2=0.0;
  for(i=0;i<L2;i++){
    if( h[i] > 0 ){
      yi=i/L;xi=i-yi*L;     //i-coordinates
      for(j=i+1;j<L2;j++){
	if( h[j] > 0 ){
	  yj=j/L; xj=j-yj*L;//j-coordinates
	  xij    = (double)(xj-xi); yij = (double)(yj-yi);
	  rij    = xij*xij + yij*yij;
	  len2  += rij;
	  norm2 += 1.0; //Count number of pairs
	  if( rij > len3) len3 = rij;
	}//if( h[j] > 0 )
      }//for(j=i+1;j<L2;j++)
    }//if( h[i] > 0 )
  }//for(i=0;i<L2;i++)
  *l2 = len2/norm2;
  *l3 = len3;
}//l2l3()

/************************  end()     *****************************/
void end(){
  //int i,x,y,Lh;
  //Lh=L/2;
  /*Print average histograms*/
  //if(measure_all){
  //  for(i=0;i<L2;i++){
  //    y=i/L;x=i-y*L;x-=Lh;y-=Lh;
  //    printf("hav %d %d %f\n",x,y,((double)hav[i])/((double)Nwalk));
  //  }//for(i=0;i<L2;i++)
  //}
}//end
/************************  drandom() *****************************/
#define a 16807
#define m 2147483647
#define q 127773
#define r 2836
#define conv (1.0/(m-1))

long seed; 

void init_random(){

  seed = (long) time( (time_t) 0);
  printf("# Initiated drandom() with seed= %d\n",seed);

}

double drandom(){
  long l;

  l    = seed/q;
  seed = a*(seed-q*l) - r*l;
  if(seed < 0) seed +=m;
  return conv*(seed-1);
}
