/********************************* rw1.c ******************************/
/*
 Simple variation of the rw.c program. In order to use it:
 1. compile:
% cc rw1.c -o rw1
 2. run and save standard output in a file
% ./rw1 <Nwalk> <Nstep>   >&  rwalk.dat
 where Nwalk the number of walks and Nstep the length N of the walk.
 3. Output: 
 The first 15 configurations are explicitly printed in order to 
 visualize them. The format is:
w <Nwalk>  <x> <y>
 so that the following command puts each configuration in a file 
 rwalk.<Nwalk>:
% grep ^w rwalk.dat | awk '{print $3,$4 > "rwalk." $2}'
 The other data printed is the final position x,y, the distance 
 squared R^2 and the number of loops n in the format:
R <Nwalk> <R^2> <x> <y> <n>
 and the average <R^2> can be computed for example by the command:
% grep ^R rwalk.dat | awk '{print $3}' | ./average
*/
#include <stdio.h>
#include <stdlib.h>

int    Nwalk, Nstep;
void   init_random(), init(int, char**);
double drandom();

int    main(int argc, char **argv){

  int    iwalk,istep,ir,n;
  int    x,y;

  init(argc,argv); // set variables, seed drandom() with time()
  for(iwalk=1;iwalk<=Nwalk;iwalk++){
    x=0;y=0;n=0;
    for(istep=1;istep<=Nstep;istep++){
      ir=(int)(drandom()*4);
      switch(ir){
      case 0:
	x ++;
	break;
      case 1:
	x --;
	break;
      case 2:
	y ++;
	break;
      case 3:
	y --;
	break;
      }//switch(ir)
      if(iwalk<=15) printf("w %d %d %d\n"   ,iwalk,x,y);
      //same as if(x==0 && y==0) n++;
      if( ! (x || y) ) n++; 
    }//istep
    printf  ("R %d %d %d %d %d\n",iwalk,x*x+y*y,x,y,n);
    fflush (stdout);
  }//iwalk
}//main

/************************  init   () *****************************/
void init(int argc, char **argv){

  if( argc != 3 ){
    fprintf(stderr,"Usage: %s <Nwalk> <Nstep>\n",basename(argv[0]));
    exit(1);
  }
  Nwalk = atoi(argv[1]);
  Nstep = atoi(argv[2]);
  printf("# Nwalk= %d random walks with Nstep= %d steps\n",Nwalk,Nstep);
  init_random();
}

/************************  drandom() *****************************/
#define a 16807
#define m 2147483647
#define q 127773
#define r 2836
#define conv (1.0/(m-1))

long seed; 

void init_random(){

  seed = (long) time( (time_t) 0);
  printf("# Initiated drandom() with seed= %d\n",seed);

}

double drandom(){
  long l;

  l    = seed/q;
  seed = a*(seed-q*l) - r*l;
  if(seed < 0) seed +=m;
  return conv*(seed-1);
}
