/************************  saw2.c *********************************/
/*
 Compile with:
 gcc -O2     saw2.c -o saw2
 icc -O3 -xK saw2.c -o saw2 (PIII - gcc is faster)
 icc -O3 -xW saw2.c -o saw2 (PIV)
 */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

/* 
   Nwalk: Number of random walks generated.
   N    : Length of random walks
   L    : Linear size of lattice and L2 = L*L  
 */
int    N, L, L2;
int    *h; /* The frequency function array h[L2]: How many times each 
              point is visited by the random walker */
long   Nwalk,iwalk; /*Random walk counter           */
void   init(int, char**),end();
void   generate_a_walk(),measure();
double measA1(); /*Area and length functions        */
void   measA2l1(double *,double *),measl2l3(double *,double *);
double drandom();
void   init_random();


/************************  main   () *****************************/
int main(int argc, char **argv){
  init(argc,argv); /*initialize program*/
  for(iwalk=1;iwalk<=Nwalk;iwalk++){
    generate_a_walk();
    //measure();
  }
  end();
} //main()


/************************  init   () *****************************/
void init(int argc, char **argv){
  int i;
  if( argc != 4 ){
    fprintf(stderr,"Usage: %s <Nwalk> <N> <L>\n",basename(argv[0]));
    exit(1);
  }
  Nwalk       = atol(argv[1]);
  N           = atoi(argv[2]);
  L           = atoi(argv[3]);
  if( L%2 == 0 ) L++;  //we accept odd L only
  L2    = L*L;
  printf("# Size of lattice: L= %d  L2= %d\n",L,L2);
  printf("# Nwalk= %ld SAW random walks with Nstep= %d steps\n",Nwalk,N);
  /*Allocate memory for the h[L2] array */
  h   = (int *)malloc((size_t) (L2*sizeof(int)));
  if(h  ==NULL){fprintf(stderr,"saw2: Not enough memory!\n");exit(1);}
  for(i=0;i<L2;i++) {h[i] = 0;} /*Init to zero*/
  init_random();/*Initialize random number generator*/
}

/************************  generate_a_walk() *********************/
void generate_a_walk(){
  int i,ir,r,r0;
  int dir;
  int boing; //count hits at the edge of lattice
  int nret ; //count no. of comeback times
  boing=0; nret=0;
  for(i=0;i<L2;i++) h[i] = 0;
  /*r is the position of the walker on the lattice*/
  r = L2/2; /*We place walker at the center of lattice (L2 is odd!)*/
  r0= r;
  dir=(int)(drandom()*4);/*dir is the stored direction of the previous step*/
  for(i=0;i<N;i++){ /*N steps foreach walk*/
    /*3 possible choices: (dir-1)%4,(dir)%4,(dir+1)%4*/
    ir=dir+3+(int)(drandom()*3);//ir always >0 for % operation, we add 4
    ir %= 4;
    dir = ir;//Stored for next move
    //The four directions are now stored counterclockwise:
    switch(ir){
    case 0: // Move in +X direction
      if((++r )>=L2){r -= L2;}//carefull ++r not r++!
      if(h[r]++){return;}
      break;
    case 2: // Move in -X direction
      if((--r )< 0 ){r += L2;}//carefull --r not r--!
      if(h[r]++){return;}
      break;
    case 1: // Move in +Y direction
      if((r+=L)>=L2){r -= L2;}
      if(h[r]++){return;}
      break;
    case 3: // Move in -Y direction
      if((r-=L)< 0 ){r += L2;}
      if(h[r]++){return;}
      break;
    }//switch(ir)
    if(i>55){printf("w %ld %d %d\n",iwalk,i,r);}//if you wish to print the path
  }//for(i=0;i<N;i++)
}//generate_a_walk
/************************  end()     *****************************/
void end(){
  exit(0);
}//end
/************************  drandom() *****************************/
#define a 16807
#define m 2147483647
#define q 127773
#define r 2836
#define conv (1.0/(m-1))

long seed; 

void init_random(){

  seed = (long) time( (time_t) 0);
  printf("# Initiated drandom() with seed= %d\n",seed);

}

double drandom(){
  long l;

  l    = seed/q;
  seed = a*(seed-q*l) - r*l;
  if(seed < 0) seed +=m;
  return conv*(seed-1);
}
