/************************ options.c ***********************************/
#include "include.h"
char prog[1024];

/*Get the options function: See "man 3 getopt" for usage*/
/*Option letters are defined with this string*/
#define OPTARGS  "hL:b:s:S:n:"
void get_the_options(int argc,char **argv){
  int c,errflg = 0;

  strcpy(prog,(char *)basename(argv[0]));
  while (!errflg && (c = getopt(argc, argv, OPTARGS)) != -1){
    switch(c){
    case 'L':
      L      = atoi(optarg);
      break;
    case 'b':
      beta   = atof(optarg);
      break;
    case 's':
      start  = atoi(optarg);
      break;
    case 'S':
      seed   = atol(optarg);
      break;
    case 'n':
      nsweep = atoi(optarg);
      break;
    case 'h':
      errflg++;/*call usage*/
      break;
    default:
      errflg++;
    }/*switch*/
    if(errflg) usage(argv);
  }/*while...*/
}/*get_the_options()*/
/* --------------------------  usage() ----------------------------- */
void usage(char **argv){
  /*Careful: New lines end with \n\ : No space after last backslacsh
    indicates line is broken....*/
  fprintf(stderr,"\
Usage: %s  [options]                                                \n\
       -L: Lattice length (N=L*L)                                   \n\
       -b: beta  (options beta overrides the one in config)         \n\
       -s: start (0 cold, 1 hot, 2 old config.)                     \n\
       -S: seed  (options seed overrides the one in config)         \n\
       -n: number of sweeps and measurements of E and M             \n\
Monte Carlo simulation of 2d Ising Model. Metropolis is used by     \n\
default. Using the   \n\
options, the parameters of the simulations must be set for a new run\n\
(start=0,1). If start=2, a configuration is read from the file conf.\n",prog);
// printed usage message "
  exit(1);
}/*usage()*/ 
/* --------------------------  locerr() ---------------------------- */
void locerr( char *errmes ){ 
  fprintf(stderr,"%s: %s Exiting....\n",prog,errmes); 
  exit(1); 
}
/* --------------------------  simmessage() ------------------------ */
/*print a message about the simulation: */
#include <time.h>
void simmessage(FILE *fp){

  char USER[100],HOST[100],MACH[100];
  time_t t;

  time(&t);/* store time in seconds in *t. see: "man 2 time"  */
  strcpy(USER,getenv("USER"));
  strcpy(MACH,getenv("HOSTTYPE"));
  strcpy(HOST,getenv("HOST"));

  fprintf(fp,"\
# ###################################################################\n\
#      2d Ising Model with Metropolis algorithm on square lattice    \n\
# Run on %s ( %s ) by %s on %s#                                      \n\
# L       = %d (Lattice linear dimension, N=L*L)                     \n\
# seed    = %ld (random number gener. seed)                          \n\
# nsweeps = %d (No. of sweeps)                                       \n\
# beta    = %f                                                       \n\
# start   = %d (0 cold, 1 hot, 2 old config)\n",
          HOST,MACH,USER,ctime(&t),
          L,seed,nsweep,beta,start);
  fflush(fp);
}/* message()*/
