/* This is an extension of the file nrutil.c of the recipes to include 
    more general definitions of tensors etc. 
    In particular:
    d3tensor, free_d3tensor: double 3 tensor
    d4tensor, free_d4tensor: double 4 tensor
    cdvector,civector,cdmatrix,cimatrix,cd3tensor,cd4tensor: Arrays initialized to 0
    slivector (signed  long vector)
    zero_dvector ,zero_ivector,zero_dmatrix,zero_imatrix,
    zero_d3vector,zero_d4tensor: Put elem. of arrays =0
    K. Anagnostopoulos, NBI Jan 96. 
 */

#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#define NR_END 1
#define FREE_ARG char*


long *slvector(long nl, long nh)
/* allocate an long vector with subscript range v[nl..nh] */
{
	long *v;

	v=(long *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(long)));
	if (!v) nrerror("allocation failure in lvector()");
	return v-nl+NR_END;
}

void free_slvector(long *v, long nl, long nh)
/* free an unsigned long vector allocated with lvector() */
{
	free((FREE_ARG) (v+nl-NR_END));
}

double ***d3tensor(long nrl, long nrh, long ncl, long nch, long ndl, long ndh)
/* allocate a double 3tensor with range t[nrl..nrh][ncl..nch][ndl..ndh] */
{
  long i,j,nrow=nrh-nrl+1,ncol=nch-ncl+1,ndep=ndh-ndl+1;
  double ***t;

  /* allocate pointers to pointers to rows */
  t=(double ***) malloc((size_t)((nrow+NR_END)*sizeof(double**)));
  if (!t) nrerror("allocation failure 1 in f3tensor()");
  t += NR_END;
  t -= nrl;
  
  /* allocate pointers to rows and set pointers to them */
  t[nrl]=(double **) malloc((size_t)((nrow*ncol+NR_END)*sizeof(double*)));
  if (!t[nrl]) nrerror("allocation failure 2 in f3tensor()");
  t[nrl] += NR_END;
  t[nrl] -= ncl;
  
  /* allocate rows and set pointers to them */
  t[nrl][ncl]=(double *) malloc((size_t)((nrow*ncol*ndep+NR_END)*sizeof(double)));
  if (!t[nrl][ncl]) nrerror("allocation failure 3 in d3tensor()");
  t[nrl][ncl] += NR_END;
  t[nrl][ncl] -= ndl;
  
  for(j=ncl+1;j<=nch;j++) t[nrl][j]=t[nrl][j-1]+ndep;
  for(i=nrl+1;i<=nrh;i++) {
    t[i]=t[i-1]+ncol;
    t[i][ncl]=t[i-1][ncl]+ncol*ndep;
    for(j=ncl+1;j<=nch;j++) t[i][j]=t[i][j-1]+ndep;
  }

  /* return pointer to array of pointers to rows */
  return t;
}/* double ***d3tensor */

double ****d4tensor(long nrl, long nrh, long ncl, long nch, long ndl, long ndh,
		    long nwl, long nwh)
/* allocate a double 4tensor with range 
     t[nrl..nrh][ncl..nch][ndl..ndh][nwl...nwh] */
{
  
  long i,j,k,nrow=nrh-nrl+1,ncol=nch-ncl+1,ndep=ndh-ndl+1,nwid=nwh-nwl+1;
  double ****t;

  t=(double ****) malloc((size_t)((nrow+NR_END)*sizeof(double***)));
  if (!t) nrerror("allocation failure 1 in d4tensor()");
  t += NR_END;
  t -= nrl;

  t[nrl]=(double ***) malloc((size_t)((nrow*ncol+NR_END)*sizeof(double**)));
  if (!t[nrl]) nrerror("allocation failure 2 in d4tensor()");
  t[nrl] += NR_END;
  t[nrl] -= ncl;

  t[nrl][ncl]=(double **) malloc((size_t)((nrow*ncol*ndep+NR_END)*sizeof(double*)));
  if (!t[nrl][ncl]) nrerror("allocation failure 3 in d4tensor()");
  t[nrl][ncl] += NR_END;
  t[nrl][ncl] -= ndl;


  t[nrl][ncl][ndl]=(double *) 
    malloc((size_t)((nrow*ncol*ndep*nwid+NR_END)*sizeof(double)));
  if (!t[nrl][ncl][ndl]) nrerror("allocation failure 4 in d4tensor()");
  t[nrl][ncl][ndl] += NR_END;
  t[nrl][ncl][ndl] -= nwl;

/* Take good care of the boundaries. In the sums, the low value of the index
   (nrl,ncl,ndl,nwl) is never assumed, so it must be taken care separately */

/* First fix low value of i=nrl */
  for(k=ndl+1;k<=ndh;k++) 
    t[nrl][ncl][k]=t[nrl][ncl][k-1]+nwid;
  for(j=ncl+1;j<=nch;j++){
    t[nrl][j]=t[nrl][j-1]+ndep; /* tun j freely */
    t[nrl][j][ndl]=t[nrl][j-1][ndl]+ndep*nwid; /*boundary*/
/*  Now run the j,k indices freely */
    for(k=ndl+1;k<=ndh;k++)
      t[nrl][j][k]=t[nrl][j][k-1]+nwid;
  }

/* Now i can run freely. In the loop we take care of low j and k */
  for(i=nrl+1;i<=nrh;i++){
    t[i]=t[i-1]+ncol;
    t[i][ncl]=t[i-1][ncl]+ncol*ndep;
    t[i][ncl][ndl]=t[i-1][ncl][ndl]+ncol*ndep*nwid;
    for(k=ndl+1;k<=ndh;k++) 
      t[i][ncl][k]=t[i][ncl][k-1]+nwid;
    for(j=ncl+1;j<=nch;j++){
      t[i][j]=t[i][j-1]+ndep;
      t[i][j][ndl]=t[i][j-1][ndl]+ndep*nwid;
      for(k=ndl+1;k<=ndh;k++)
	t[i][j][k]=t[i][j][k-1]+nwid;
    }
  }


  return t;
}/* double ****d4tensor */

void free_d3tensor(double ***t, long nrl, long nrh, long ncl, long nch,
	long ndl, long ndh)
/* free a float d3tensor allocated by d3tensor() */
{
  free((FREE_ARG) (t[nrl][ncl]+ndl-NR_END));
  free((FREE_ARG) (t[nrl]+ncl-NR_END));
  free((FREE_ARG) (t+nrl-NR_END));
}

void free_d4tensor(double ****t,
		   long nrl, long nrh, long ncl, long nch, long ndl, long ndh,
		   long nwl, long nwh)
/* free a float d4tensor allocated by d4tensor() */
{
  free((FREE_ARG) (t[nrl][ncl][nwl]+nwl-NR_END));
  free((FREE_ARG) (t[nrl][ncl]+ndl-NR_END));
  free((FREE_ARG) (t[nrl]+ncl-NR_END));
  free((FREE_ARG) (t+nrl-NR_END));
}

/****************  c* functions   ************************************/
/*******   This part defines functions that initilize arrays to zero */



double *cdvector(long nl, long nh)
/* allocate a double vector with subscript range v[nl..nh] */
{
	double *v;
	int i;

	v=(double *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(double)));
	if (!v) nrerror("allocation failure in dvector()");
	for(i=nl;i<=nh;i++)v[i]=0.0;
	return v-nl+NR_END;
}


int *civector(long nl, long nh)
/* allocate an int vector with subscript range v[nl..nh] */
{
	int *v,i;

	v=(int *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(int)));
	if (!v) nrerror("allocation failure in ivector()");
	for(i=nl;i<=nh;i++)v[i]=0;
	return v-nl+NR_END;
}

double **cdmatrix(long nrl, long nrh, long ncl, long nch)
/* allocate a double matrix with subscript range m[nrl..nrh][ncl..nch] */
{
	long i,j, nrow=nrh-nrl+1,ncol=nch-ncl+1;
	double **m;

	/* allocate pointers to rows */
	m=(double **) malloc((size_t)((nrow+NR_END)*sizeof(double*)));
	if (!m) nrerror("allocation failure 1 in matrix()");
	m += NR_END;
	m -= nrl;

	/* allocate rows and set pointers to them */
	m[nrl]=(double *) malloc((size_t)((nrow*ncol+NR_END)*sizeof(double)));
	if (!m[nrl]) nrerror("allocation failure 2 in matrix()");
	m[nrl] += NR_END;
	m[nrl] -= ncl;

	for(i=nrl+1;i<=nrh;i++) m[i]=m[i-1]+ncol;

	for(i=nrl;i<=nrh;i++)
	  for(j=ncl;j<=nch;j++)
	    m[i][j] = 0.0;

	/* return pointer to array of pointers to rows */
	return m;
}
int **cimatrix(long nrl, long nrh, long ncl, long nch)
/* allocate a int matrix with subscript range m[nrl..nrh][ncl..nch] */
{
	long i,j, nrow=nrh-nrl+1,ncol=nch-ncl+1;
	int **m;

	/* allocate pointers to rows */
	m=(int **) malloc((size_t)((nrow+NR_END)*sizeof(int*)));
	if (!m) nrerror("allocation failure 1 in matrix()");
	m += NR_END;
	m -= nrl;


	/* allocate rows and set pointers to them */
	m[nrl]=(int *) malloc((size_t)((nrow*ncol+NR_END)*sizeof(int)));
	if (!m[nrl]) nrerror("allocation failure 2 in matrix()");
	m[nrl] += NR_END;
	m[nrl] -= ncl;

	for(i=nrl+1;i<=nrh;i++) m[i]=m[i-1]+ncol;

	for(i=nrl;i<=nrh;i++)
	  for(j=ncl;j<=nch;j++)
	    m[i][j] = 0;

	/* return pointer to array of pointers to rows */
	return m;
}

double ***cd3tensor(long nrl, long nrh, long ncl, long nch, long ndl, long ndh)
/* allocate a double 3tensor with range t[nrl..nrh][ncl..nch][ndl..ndh] */
{
  long i,j,k,nrow=nrh-nrl+1,ncol=nch-ncl+1,ndep=ndh-ndl+1;
  double ***t;

  /* allocate pointers to pointers to rows */
  t=(double ***) malloc((size_t)((nrow+NR_END)*sizeof(double**)));
  if (!t) nrerror("allocation failure 1 in f3tensor()");
  t += NR_END;
  t -= nrl;
  
  /* allocate pointers to rows and set pointers to them */
  t[nrl]=(double **) malloc((size_t)((nrow*ncol+NR_END)*sizeof(double*)));
  if (!t[nrl]) nrerror("allocation failure 2 in f3tensor()");
  t[nrl] += NR_END;
  t[nrl] -= ncl;
  
  /* allocate rows and set pointers to them */
  t[nrl][ncl]=(double *) malloc((size_t)((nrow*ncol*ndep+NR_END)*sizeof(double)));
  if (!t[nrl][ncl]) nrerror("allocation failure 3 in d3tensor()");
  t[nrl][ncl] += NR_END;
  t[nrl][ncl] -= ndl;
  
  for(j=ncl+1;j<=nch;j++) t[nrl][j]=t[nrl][j-1]+ndep;
  for(i=nrl+1;i<=nrh;i++) {
    t[i]=t[i-1]+ncol;
    t[i][ncl]=t[i-1][ncl]+ncol*ndep;
    for(j=ncl+1;j<=nch;j++) t[i][j]=t[i][j-1]+ndep;
  }

  for(i=nrl;i<=nrh;i++)
    for(j=ncl;j<=nch;j++)
      for(k=ndl;k<=ndh;k++)
	t[i][j][k] = 0.0;
  /* return pointer to array of pointers to rows */
  return t;
}/* double ***d3tensor */

double ****cd4tensor(long nrl, long nrh, long ncl, long nch, long ndl, long ndh,
		    long nwl, long nwh)
/* allocate a double 4tensor with range 
     t[nrl..nrh][ncl..nch][ndl..ndh][nwl...nwh] 
   Initialize entries to 0 */
{
  
  long i,j,k,l,nrow=nrh-nrl+1,ncol=nch-ncl+1,ndep=ndh-ndl+1,nwid=nwh-nwl+1;
  double ****t;

  t=(double ****) malloc((size_t)((nrow+NR_END)*sizeof(double***)));
  if (!t) nrerror("allocation failure 1 in d4tensor()");
  t += NR_END;
  t -= nrl;

  t[nrl]=(double ***) malloc((size_t)((nrow*ncol+NR_END)*sizeof(double**)));
  if (!t[nrl]) nrerror("allocation failure 2 in d4tensor()");
  t[nrl] += NR_END;
  t[nrl] -= ncl;

  t[nrl][ncl]=(double **) malloc((size_t)((nrow*ncol*ndep+NR_END)*sizeof(double*)));
  if (!t[nrl][ncl]) nrerror("allocation failure 3 in d4tensor()");
  t[nrl][ncl] += NR_END;
  t[nrl][ncl] -= ndl;


  t[nrl][ncl][ndl]=(double *) 
    malloc((size_t)((nrow*ncol*ndep*nwid+NR_END)*sizeof(double))); 
  if (!t[nrl][ncl][ndl]) nrerror("allocation failure 4 in d4tensor()");
  t[nrl][ncl][ndl] += NR_END;
  t[nrl][ncl][ndl] -= nwl;

/* Take good care of the boundaries. In the sums, the low value of the index
   (nrl,ncl,ndl,nwl) is never assumed, so it must be taken care separately */

/* First fix low value of i=nrl */
  for(k=ndl+1;k<=ndh;k++) 
    t[nrl][ncl][k]=t[nrl][ncl][k-1]+nwid;
  for(j=ncl+1;j<=nch;j++){
    t[nrl][j]=t[nrl][j-1]+ndep; /* tun j freely */
    t[nrl][j][ndl]=t[nrl][j-1][ndl]+ndep*nwid; /*boundary*/
/*  Now run the j,k indices freely */
    for(k=ndl+1;k<=ndh;k++)
      t[nrl][j][k]=t[nrl][j][k-1]+nwid;
  }

/* Now i can run freely. In the loop we take care of low j and k */
  for(i=nrl+1;i<=nrh;i++){
    t[i]=t[i-1]+ncol;
    t[i][ncl]=t[i-1][ncl]+ncol*ndep;
    t[i][ncl][ndl]=t[i-1][ncl][ndl]+ncol*ndep*nwid;
    for(k=ndl+1;k<=ndh;k++) 
      t[i][ncl][k]=t[i][ncl][k-1]+nwid;
    for(j=ncl+1;j<=nch;j++){
      t[i][j]=t[i][j-1]+ndep;
      t[i][j][ndl]=t[i][j-1][ndl]+ndep*nwid;
      for(k=ndl+1;k<=ndh;k++)
	t[i][j][k]=t[i][j][k-1]+nwid;
    }
  }

  for(i=nrl;i<=nrh;i++)
    for(j=ncl;j<=nch;j++)
      for(k=ndl;k<=ndh;k++)
	for(l=nwl;l<=nwh;l++)
	  t[i][j][k][l] = 0.0;

  return t;
}/* double ****d4tensor */
  
/**************** Functions that zero elements of arrays *****************/

void zero_dvector(double *v, long nl, long nh){
  int i;

  for(i=nl;i<=nh;i++)
    v[i] = 0.0;
}

void zero_ivector(int *v, long nl, long nh){
  int i;

  for(i=nl;i<=nh;i++)
    v[i] = 0;
}

void zero_dmatrix(double **v, long nrl, long nrh, long ncl, long nch){
  int i,j;

  for(i=nrl;i<=nrh;i++)
    for(j=ncl;j<=nch;j++)
      v[i][j] = 0.0;
}

void zero_imatrix(int **v, long nrl, long nrh, long ncl, long nch){
  int i,j;

  for(i=nrl;i<=nrh;i++)
    for(j=ncl;j<=nch;j++)
      v[i][j] = 0;
}

void zero_d3tensor(double ***v, long nrl, long nrh, long ncl, long nch, 
		   long ndl, long ndh){
  int i,j,k;

  for(i=nrl;i<=nrh;i++)
    for(j=ncl;j<=nch;j++)
      for(k=ndl;k<=ndh;k++)
	v[i][j][k] = 0.0;
}

void zero_d4tensor(double ****v, long nrl, long nrh, long ncl, long nch, 
		   long ndl, long ndh, long nwl, long nwh){
  int i,j,k,l;

  for(i=nrl;i<=nrh;i++)
    for(j=ncl;j<=nch;j++)
      for(k=ndl;k<=ndh;k++)
	for(l=nwl;l<=nwh;l++)
	  v[i][j][k][l] = 0.0;

}
