CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C         Subtract-and-borrow random number generator proposed by
C         Marsaglia and Zaman, implemented by F. James with the name
C         RCARRY in 1991, and later improved by Martin Luescher
C         in 1993 to produce "Luxury Pseudorandom Numbers".
C     Fortran 77 coded by F. James, 1993
C
C   The excellent statistical properties are due to its close relation to the
C   Kolmogorov K-system originally proposed by Savvidy et.al
C
C   Simple user interface added by K. Anagnostopoulos, 1999
C
C   References: 
C
C  [1] G.K. Savvidy and N.G. Ter-Arutyunyan Savvidy, EPI-865-16-86
C      (1986); J. Comput. Phys. 97 (1991) 566. 
C  [2] N.Z. Akopov,  G.K. Savvidy and
C      N.G. Ter-Arutyunyan Savvidy, J. Comput. Phys. 97 (1991) 573.
C  [3] M. Luscher, Comput. Phys. Commun. 79 (1994) 100.
C  [4] F. James, Comput. Phys. Commun. 79 (1994) 111; 
C      {\it Erratum} 97 (1996) 357.
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CC
CC   RANLUX and interfaces to it:
CC
CC   ranf()   - uniform (0,1) -- 0,1, not included
CC   granf()  - ~exp(-x^2  ) s=1/sqrt(2)
CC   granf1() - ~exp(-x^2/2) s=1
CC   gasdev() - ~exp(-x^2/2) s=1
CC   ranfl() - log(ranf()) (x=-ranfl()~ exp(-x), <x>=1,s_x=1)
CC   iranx(N) - uniform integer in [1,N] -- 1,N included
CC
CC   Choice between granfs and gasdev is machine dependent
CC
CC   If you define the macro OOGL during compilation, no output 
CC   is produced
CC
CC
CC   K. Anagnostopoulos, NBI 3/99
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


CC C  A test program explaining the user interface
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CC       program testit
CC       implicit none

CC C  -  Declarations needed in all programs:
CC       integer iseed,ranlux_lev
CC       common/ran_seed/iseed,ranlux_lev
CC       double precision ranf,ranfl,granf,gasdev,granf1
CC       integer iranx
CC C  ---------------------------------------

CC C  -  local declarations:
CC       logical new
CC       integer i,ir
CC       double precision x,logx,normx

CC C  -  seeds the random number
CC       iseed=12345
CC C  -  start generator from the beginning or from other seed
CC       new=.true.
CC c      new=.false.  !we read seed from file aaa.iseed
CC c      ranlux_lev=2 !set the level of ranlux (default 2)
CC       call initranf(new)

CC C  -- This is how we generate the numbers:
CC       do i=1,50000
CC c       x =ranf()      ! x uniform in (0,1) - not 1 and 0
CC c      ir    =iranx(1000) ! ir between 1,1000 inclusive
CC        x  =ranfl()    ! log of random number
CC c       x =granf()     ! normally distributed ~exp(-x^2) s=1/sqrt(2)
CC c       x =granf1()     ! normally distributed ~exp(-x^/2) s=1
CC C -   from recipes
CC C -   y~ 1/Sqrt(2 Pi) exp(-y^2/2) => <y>=0, s_y=1
CC C -   If z=Sqrt(s) y + mu => <z>=mu, s_z=sqrt(s)
CC c       x = gasdev()   ! normally distributed 
CC c       call ranlux(x,1)   ! simple call to ranlux (slower)
CC        print *, x
CC       enddo

CC C  -- we save the status of generator in file aaa.seed
CC       call saveranf

CC       stop
CC       end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  -                    TESTS 
C  
C  -  Ranlux level 2:
C  -  Times(cpu sec) on hetws14: 433MHz alpha generating 10000000
C      numbers and hetws17: 350MHz pentium
CCC         hetws14    hetws17   
CCC ranlux  3.78        5.98
CCC ranf    2.29        4.74  (65% and 26% gain respectively comp to ranlux)
CCC granf   4.54       20.85 
CCC granf1  6.64       17.34
CCC gasdev  6.86       13.10
CCC iranx   2.28        7.02
CCC ranfl   2.44       10.63
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C--------------------------------------------------------------------
      function ranf()
      implicit none
      logical flaga
      double precision ranf
      integer iseed,ranlux_lev
      common/ran_seed/ iseed,ranlux_lev
      integer iranf_siz
      parameter (iranf_siz=1000) !careful, must be the same as in initranf
      double precision random(iranf_siz)
      common /ranlux_rand/random
      integer iran_pos
      data iran_pos /1/
      logical notyet
      data notyet /.TRUE./
      double precision x

c 39   CONTINUE
      if(iran_pos.LE.iranf_siz)then
       ranf=random(iran_pos)
       iran_pos=iran_pos+1
      else
       call ranlux(random,iranf_siz)
       ranf=random(1)
       iran_pos=2
      endif
c      Hopefully not needed!
c      if(x.le.0.0 .or. x.ge.1.0) goto 39
c      ranf = x


      return
      end

C--------------------------------------------------------------------
      subroutine saveranf
      INTEGER restart_seed(25)

      open(unit=44,file='aaa.seed',status='unknown',form='formatted')
      CALL RLUXUT(restart_seed) !save the seed of ranlux
      write(44,400)restart_seed
      close(44)
 400  FORMAT(1x, 5I15)
      return
      end

C ----- interface to initranf
      subroutine initrandom(seed,rlev)
      implicit none
      integer seed,rlev
      integer iseed,ranlux_lev
      common/ran_seed/ iseed,ranlux_lev
      logical new_seed
      if(seed.gt.0)then
       new_seed=.TRUE.
       iseed = seed
       ranlux_lev = rlev
      else
       new_seed=.FALSE.
      endif
      call initranf(new_seed)
      end
       

C--------------------------------------------------------------------
      subroutine initranf(new_seed)
      implicit none
      logical new_seed
      logical flaga
      integer iseed,ranlux_lev
      common/ran_seed/ iseed,ranlux_lev
      integer iranf_siz
      parameter (iranf_siz=1000)
      double precision random(iranf_siz)
      common /ranlux_rand/random
      INTEGER restart_seed(25)  !array for restarting ranlux

      if(ranlux_lev.lt.1.or.ranlux_lev.gt.4)ranlux_lev=2
      if(new_seed)then
       CALL RLUXGO(ranlux_lev,abs(iseed),0,0)
       call ranlux(random,iranf_siz)
      else
       open(unit=44,file='aaa.iseed',status='old',form='formatted')
       read(44,400)restart_seed
       close(44)
       call RLUXIN(restart_seed) !initialize ranlux
       call ranlux(random,iranf_siz)
      endif

 400  FORMAT(1x, 5I15)
      return
      end


c*************************************************************
      function iranx(n)
      implicit none
      integer iranx,n
      double precision ranf
c
c-------return integer value between 1 and n, both included
c
c
      iranx=1+int(n*ranf())
c     Ranlux does not return 0 or 1
c      if(iranx.gt.n)iranx=n
c      if(iranx.lt.1)iranx=1
      return
      end
c*************************************************************

c*************************************************************
      function ranfl()
      implicit none
      double precision ranfl,x,ranf
c
c     returns log of random number
c
      x=ranf()
c      if(x.eq.0.)x=1.d0 - not possible from ranlux
c     Ranlux does not return 0
      ranfl=dlog(x)
      return
      end

c*************************************************************
      function granf()
      implicit none
      double precision granf,pi,ranf,ranfl
c
c     returns normal distribution ~ exp(-x^2), sigma=1/sqrt(2)
c
      data pi/3.14159265358979323846/
      granf=sqrt(-dlog(ranf()))*cos(pi*ranf())
      return
      end
c*************************************************************

c*************************************************************
      function granf1()
      implicit none
      double precision granf1,pi,ranf,ranfl
      double precision y1,y2,x1,x2,gset
      integer iset
      DATA iset /0/
      SAVE iset,gset
c
c     returns normal distribution ~ exp(-x^2/2), sigma=1
c
      PARAMETER(pi=3.14159265358979323846)
c      granf=sqrt(-ranfl())*cos(pi*ranf())
      if (iset.eq.0) then
       x1=ranf()
       x2=ranf()
       granf1=dsqrt(-2.0d0*dlog(x1))*dcos(2.0d0*pi*x2)
       gset  =dsqrt(-2.0d0*dlog(x1))*dsin(2.0d0*pi*x2)
       iset  =1
      else
       granf1=gset
       iset  =0
      endif

      return
      end
c*************************************************************

C -   from recipes
C -   y~ 1/Sqrt(2 Pi) exp(-y^2/2) => <y>=0, s_y=1
C -   If z=Sqrt(s) y + mu => <z>=mu, s_z=sqrt(s)
      FUNCTION gasdev()
      implicit none
      double precision gasdev,ranf

      INTEGER iset
      double precision fac,gset,rsq,v1,v2,ran1
      SAVE iset,gset
      DATA iset/0/
      if (iset.eq.0) then
1       v1=2.d0*ranf()-1.d0
        v2=2.d0*ranf()-1.d0
        rsq=v1*v1+v2*v2
        if(rsq.ge.1.d0.or.rsq.eq.0.d0)goto 1
        fac=dsqrt(-2.d0*dlog(rsq)/rsq)
        gset=v1*fac
        gasdev=v2*fac
        iset=1
      else
        gasdev=gset
        iset=0
      endif
      return
      END



CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CC
CC      DOUBLE PRECISION VERSION OF RANLUX.
CC
CC      I ADDED:
CC      IMPLICIT  REAL * 8 (A-H) , REAL * 8 (O - Z)
CC      AND CHANGED SOME REAL() CONVERSIONS TO DBLE()
CC
CC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC



CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCC  See end of file for sample program on how to use this routine CC
CCCCC  Check out comments on the luxury level.                       CC
CCCCC  Default luxury level set in include.h                         CC
CCCCC  (parameter local_lux_level)                                   CC
CCCCC                                                                CC
CCCCC  K. Anagnostopoulos, NBI Nov 95                                CC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      SUBROUTINE RANLUX(RVEC,LENV)
C         Subtract-and-borrow random number generator proposed by
C         Marsaglia and Zaman, implemented by F. James with the name
C         RCARRY in 1991, and later improved by Martin Luescher
C         in 1993 to produce "Luxury Pseudorandom Numbers".
C     Fortran 77 coded by F. James, 1993
C
C   LUXURY LEVELS.
C   ------ ------      The available luxury levels are:
C
C  level 0  (p=24): equivalent to the original RCARRY of Marsaglia
C           and Zaman, very long period, but fails many tests.
C  level 1  (p=48): considerable improvement in quality over level 0,
C           now passes the gap test, but still fails spectral test.
C  level 2  (p=97): passes all known tests, but theoretically still
C           defective.
C  level 3  (p=223): DEFAULT VALUE.  Any theoretically possible
C           correlations have very small chance of being observed.
C  level 4  (p=389): highest possible luxury, all 24 bits chaotic.
C
C!!! ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C!!!  Calling sequences for RANLUX:                                  ++
C!!!      CALL RANLUX (RVEC, LEN)   returns a vector RVEC of LEN     ++
C!!!                   32-bit random floating point numbers between  ++
C!!!                   zero (not included) and one (also not incl.). ++
C!!!      CALL RLUXGO(LUX,INT,K1,K2) initializes the generator from  ++
C!!!               one 32-bit integer INT and sets Luxury Level LUX  ++
C!!!               which is integer between zero and MAXLEV, or if   ++
C!!!               LUX .GT. 24, it sets p=LUX directly.  K1 and K2   ++
C!!!               should be set to zero unless restarting at a break++
C!!!               point given by output of RLUXAT (see RLUXAT).     ++
C!!!      CALL RLUXAT(LUX,INT,K1,K2) gets the values of four integers++
C!!!               which can be used to restart the RANLUX generator ++
C!!!               at the current point by calling RLUXGO.  K1 and K2++
C!!!               specify how many numbers were generated since the ++
C!!!               initialization with LUX and INT.  The restarting  ++
C!!!               skips over  K1+K2*E9   numbers, so it can be long.++
C!!!   A more efficient but less convenient way of restarting is by: ++
C!!!      CALL RLUXIN(ISVEC)    restarts the generator from vector   ++
C!!!                   ISVEC of 25 32-bit integers (see RLUXUT)      ++
C!!!      CALL RLUXUT(ISVEC)    outputs the current values of the 25 ++
C!!!                 32-bit integer seeds, to be used for restarting ++
C!!!      ISVEC must be dimensioned 25 in the calling program        ++
C!!! ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CC 
CC    Comments on the Luxury level:
CC    
CC    For a run on Ising coupled to gravity with spins on 1000 nodes and
CC    beta = 2.34, 10000 therm sweeps and 100 measurements the CPU time
CC    was (in secs):
CC
CC    RAN   :           247 
CC    RANLUX:  level 0  263
CC             level 1  277
CC             level 2  327    ---> same level as ranmar
CC             level 3  437
CC             level 4  609
CC
CC    The routine knows the luxury level from the seed dump in the
CC    output configuration file aaa.oco and sets it to it.
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      IMPLICIT  REAL * 8 (A-H) , REAL * 8 (O - Z)
      DIMENSION RVEC(LENV)
      DIMENSION SEEDS(24), ISEEDS(24), ISDEXT(25)
      PARAMETER (MAXLEV=4, LXDFLT= 1)!LXDFLT is the default value
      DIMENSION NDSKIP(0:MAXLEV)
      DIMENSION NEXT(24)
      PARAMETER (TWOP12=4096., IGIGA=1000000000,JSDFLT=314159265)
      PARAMETER (ITWO24=2**24, ICONS=2147483563)
      SAVE NOTYET, I24, J24, CARRY, SEEDS, TWOM24, TWOM12, LUXLEV
      SAVE NSKIP, NDSKIP, IN24, NEXT, KOUNT, MKOUNT, INSEED
      INTEGER LUXLEV
      LOGICAL NOTYET
      DATA NOTYET, LUXLEV, IN24, KOUNT, MKOUNT /.TRUE., LXDFLT, 0,0,0/
      DATA I24,J24,CARRY/24,10,0./


C                               default
C  Luxury Level   0     1     2   *3*    4
      DATA NDSKIP/0,   24,   73,  199,  365 /
Corresponds to p=24    48    97   223   389
C     time factor 1     2     3     6    10   on slow workstation
C                 1    1.5    2     3     5   on fast mainframe
C
C  NOTYET is .TRUE. if no initialization has been performed yet.
C              Default Initialization by Multiplicative Congruential
      IF (NOTYET) THEN
         NOTYET = .FALSE.
         JSEED = JSDFLT
         INSEED = JSEED
#ifndef OOGL
         WRITE(6,'(A,I12)') '# RANLUX DEFAULT INITIALIZATION: ',JSEED
#endif
         LUXLEV = LXDFLT
         NSKIP = NDSKIP(LUXLEV)
         LP = NSKIP + 24
         IN24 = 0
         KOUNT = 0
         MKOUNT = 0
#ifndef OOGL
         WRITE(6,'(A,I2,A,I4)')  '# RANLUX DEFAULT LUXURY LEVEL =  ',
     +        LUXLEV,'      p =',LP
#endif
            TWOM24 = 1.
         DO 25 I= 1, 24
            TWOM24 = TWOM24 * 0.5
         K = JSEED/53668
         JSEED = 40014*(JSEED-K*53668) -K*12211
         IF (JSEED .LT. 0)  JSEED = JSEED+ICONS
         ISEEDS(I) = MOD(JSEED,ITWO24)
   25    CONTINUE
         TWOM12 = TWOM24 * 4096.
         DO 50 I= 1,24
         SEEDS(I) = DBLE(ISEEDS(I))*TWOM24
         NEXT(I) = I-1
   50    CONTINUE
         NEXT(1) = 24
         I24 = 24
         J24 = 10
         CARRY = 0.
         IF (SEEDS(24) .EQ. 0.) CARRY = TWOM24
      ENDIF
C
C          The Generator proper: "Subtract-with-borrow",
C          as proposed by Marsaglia and Zaman,
C          Florida State University, March, 1989
C
      DO 100 IVEC= 1, LENV
      UNI = SEEDS(J24) - SEEDS(I24) - CARRY
      IF (UNI .LT. 0.)  THEN
         UNI = UNI + 1.0
         CARRY = TWOM24
      ELSE
         CARRY = 0.
      ENDIF
      SEEDS(I24) = UNI
      I24 = NEXT(I24)
      J24 = NEXT(J24)
      RVEC(IVEC) = UNI
C  small numbers (with less than 12 "significant" bits) are "padded".
      IF (UNI .LT. TWOM12)  THEN
         RVEC(IVEC) = RVEC(IVEC) + TWOM24*SEEDS(J24)
C        and zero is forbidden in case someone takes a logarithm
         IF (RVEC(IVEC) .EQ. 0.)  RVEC(IVEC) = TWOM24*TWOM24
      ENDIF
C        Skipping to luxury.  As proposed by Martin Luscher.
      IN24 = IN24 + 1
      IF (IN24 .EQ. 24)  THEN
         IN24 = 0
         KOUNT = KOUNT + NSKIP
         DO 90 ISK= 1, NSKIP
         UNI = SEEDS(J24) - SEEDS(I24) - CARRY
         IF (UNI .LT. 0.)  THEN
            UNI = UNI + 1.0
            CARRY = TWOM24
         ELSE
            CARRY = 0.
         ENDIF
         SEEDS(I24) = UNI
         I24 = NEXT(I24)
         J24 = NEXT(J24)
   90    CONTINUE
      ENDIF
  100 CONTINUE
      KOUNT = KOUNT + LENV
      IF (KOUNT .GE. IGIGA)  THEN
         MKOUNT = MKOUNT + 1
         KOUNT = KOUNT - IGIGA
      ENDIF
      RETURN
C
C           Entry to input and float integer seeds from previous run
      ENTRY RLUXIN(ISDEXT)
         TWOM24 = 1.
         DO 195 I= 1, 24
         NEXT(I) = I-1
         TWOM24 = TWOM24 * 0.5
 195     CONTINUE
         NEXT(1) = 24
         TWOM12 = TWOM24 * 4096.
#ifndef OOGL
      WRITE(6,'(A)') '# FULL INITIALIZATION OF RANLUX WITH 25 INTEGERS:'
      WRITE(6,'(\'# \',5X,5I12)') ISDEXT
#endif
CC   This line was missing BUGGGGG! K.Anagnostopoulos Apr 4 1996.
      NOTYET = .FALSE.
      DO 200 I= 1, 24
      SEEDS(I) = DBLE(ISDEXT(I))*TWOM24
  200 CONTINUE
      CARRY = 0.
      IF (ISDEXT(25) .LT. 0)  CARRY = TWOM24
      ISD = IABS(ISDEXT(25))
      I24 = MOD(ISD,100)
      ISD = ISD/100
      J24 = MOD(ISD,100)
      ISD = ISD/100
      IN24 = MOD(ISD,100)
      ISD = ISD/100
      LUXLEV = ISD
        IF (LUXLEV .LE. MAXLEV) THEN
          NSKIP = NDSKIP(LUXLEV)
#ifndef OOGL
          WRITE (6,'(A,I2)') '# RANLUX LUXURY LEVEL SET BY RLUXIN TO: ',
     +                         LUXLEV
#endif
        ELSE  IF (LUXLEV .GE. 24) THEN
          NSKIP = LUXLEV - 24
#ifndef OOGL
          WRITE (6,'(A,I5)') '# RANLUX P-VALUE SET BY RLUXIN TO:',LUXLEV
#endif
        ELSE
          NSKIP = NDSKIP(MAXLEV)
          WRITE (6,'(A,I5)') '# RANLUX ILLEGAL LUXURY RLUXIN: ',LUXLEV
          LUXLEV = MAXLEV
        ENDIF
      INSEED = -1
      RETURN
C
C                    Entry to ouput seeds as integers
      ENTRY RLUXUT(ISDEXT)
      DO 300 I= 1, 24
         ISDEXT(I) = INT(SEEDS(I)*TWOP12*TWOP12)
  300 CONTINUE
      ISDEXT(25) = I24 + 100*J24 + 10000*IN24 + 1000000*LUXLEV
      IF (CARRY .GT. 0.)  ISDEXT(25) = -ISDEXT(25)
      RETURN
C
C                    Entry to output the "convenient" restart point
      ENTRY RLUXAT(LOUT,INOUT,K1,K2)
      LOUT = LUXLEV
      INOUT = INSEED
      K1 = KOUNT
      K2 = MKOUNT
      RETURN
C
C                    Entry to initialize from one or three integers
      ENTRY RLUXGO(LUX,INS,K1,K2)
         IF (LUX .LT. 0) THEN
            LUXLEV = LXDFLT
         ELSE IF (LUX .LE. MAXLEV) THEN
            LUXLEV = LUX
         ELSE IF (LUX .LT. 24 .OR. LUX .GT. 2000) THEN
            LUXLEV = MAXLEV
            WRITE (6,'(A,I7)') '# RANLUX ILLEGAL LUXURY RLUXGO: ',LUX
         ELSE
            LUXLEV = LUX
            DO 310 ILX= 0, MAXLEV
              IF (LUX .EQ. NDSKIP(ILX)+24)  LUXLEV = ILX
  310       CONTINUE
         ENDIF
      IF (LUXLEV .LE. MAXLEV)  THEN
         NSKIP = NDSKIP(LUXLEV)
#ifndef OOGL
         WRITE(6,'(A,I2,A,I4)') '# RANLUX LUXURY LEVEL SET BY RLUXGO :',
     +        LUXLEV,'     P=', NSKIP+24
#endif
      ELSE
          NSKIP = LUXLEV - 24
#ifndef OOGL
          WRITE (6,'(A,I5)') '# RANLUX P-VALUE SET BY RLUXGO TO:',LUXLEV
#endif
      ENDIF
      IN24 = 0
      IF (INS .LT. 0)  WRITE (6,'(A)')
     +   '# Illegal initialization by RLUXGO, negative input seed'
      IF (INS .GT. 0)  THEN
        JSEED = INS
#ifndef OOGL
        WRITE(6,'(A,3I12)') '# RANLUX INITIALIZED BY RLUXGO FROM SEEDS',
     +      JSEED, K1,K2
#endif
      ELSE
        JSEED = JSDFLT
#ifndef OOGL
        WRITE(6,'(A)')'# RANLUX INITIALIZED BY RLUXGO FROM DEFAULT SEED'
#endif
      ENDIF
      INSEED = JSEED
      NOTYET = .FALSE.
      TWOM24 = 1.
         DO 325 I= 1, 24
           TWOM24 = TWOM24 * 0.5
         K = JSEED/53668
         JSEED = 40014*(JSEED-K*53668) -K*12211
         IF (JSEED .LT. 0)  JSEED = JSEED+ICONS
         ISEEDS(I) = MOD(JSEED,ITWO24)
  325    CONTINUE
      TWOM12 = TWOM24 * 4096.
         DO 350 I= 1,24
         SEEDS(I) = DBLE(ISEEDS(I))*TWOM24
         NEXT(I) = I-1
  350    CONTINUE
      NEXT(1) = 24
      I24 = 24
      J24 = 10
      CARRY = 0.
      IF (SEEDS(24) .EQ. 0.) CARRY = TWOM24
C        If restarting at a break point, skip K1 + IGIGA*K2
C        Note that this is the number of numbers delivered to
C        the user PLUS the number skipped (if luxury .GT. 0).
      KOUNT = K1
      MKOUNT = K2
      IF (K1+K2 .NE. 0)  THEN
        DO 500 IOUTER= 1, K2+1
          INNER = IGIGA
          IF (IOUTER .EQ. K2+1)  INNER = K1
          DO 450 ISK= 1, INNER
            UNI = SEEDS(J24) - SEEDS(I24) - CARRY
            IF (UNI .LT. 0.)  THEN
               UNI = UNI + 1.0
               CARRY = TWOM24
            ELSE
               CARRY = 0.
            ENDIF
            SEEDS(I24) = UNI
            I24 = NEXT(I24)
            J24 = NEXT(J24)
  450     CONTINUE
  500   CONTINUE
C         Get the right value of IN24 by direct calculation
        IN24 = MOD(KOUNT, NSKIP+24)
        IF (MKOUNT .GT. 0)  THEN
           IZIP = MOD(IGIGA, NSKIP+24)
           IZIP2 = MKOUNT*IZIP + IN24
           IN24 = MOD(IZIP2, NSKIP+24)
        ENDIF
C       Now IN24 had better be between zero and 23 inclusive
        IF (IN24 .GT. 23) THEN
           WRITE (6,'(A/A,3I11,A,I5)')
     +    '  Error in RESTARTING with RLUXGO:','  The values', INS,
     +     K1, K2, ' cannot occur at luxury level', LUXLEV
           IN24 = 0
        ENDIF
      ENDIF
      RETURN
      END


CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CC 
CC   Sample program demonstrating a simple usage of the routine.
CC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CC 
CC       PROGRAM RUN_RANLUX
CC       
CC       INTEGER mode,i
CC CCCC Careful, must be INTEGER * 4
CC       INTEGER SEED, RESTART_SEED(25)
CC 
CC CCCC This is your random number. Must be REAL*4
CC       REAL *4 random_no
CC 
CC 
CC       PRINT*,'Enter mode (0 new,1old) '
CC       READ(5,*)mode
CC 
CC       IF( mode .EQ. 0)THEN
CC 
CC CCCC  Initialization for the first time
CC         SEED = 12343543
CC CCCC  4 is the max level available here. The two last numbers must be 0.
CC         CALL RLUXGO(4,SEED,0,0)
CC 
CC       ELSE
CC 
CC CCCC  Read previous data from file input to continue from previous point
CC         OPEN(23,FILE='input',STATUS='UNKNOWN')
CC         READ(23,*)(RESTART_SEED(i),i=1,25)
CC         CALL RLUXIN(RESTART_SEED)
CC 
CC       ENDIF
CC          
CC       DO i = 1,5000
CC CCCC  Generate random  numbers
CC         CALL RANLUX(random_no,1)
CC         print*,DBLE(random_no)
CC       ENDDO
CC 
CC       OPEN(24,FILE='ouput',STATUS='UNKNOWN')
CC CCCC Save the seed to start another time
CC       CALL RLUXUT(RESTART_SEED)
CC       write(24,*)RESTART_SEED
CC       CLOSE(24)
CC 
CC       STOP
CC 
CC       END
CC 
CC 
CC 
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC



C__________________________________________________________________________
